package sample;
//
/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2012, SAS Institute Inc., Cary, NC
 *************************************************************************/
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sas.hls.client.Session;
import com.sas.hls.client.SessionFactory;
import com.sas.hls.client.exception.ConfigurationException;
import com.sas.hls.client.exception.HttpException;
import com.sas.hls.common.Attribute;
import com.sas.hls.common.exception.IllegalPathException;
import com.sas.hls.drug.analysis.Analysis;
import com.sas.hls.drug.analysis.exception.AnalysisExistsException;
import com.sas.hls.drug.analysis.exception.AnalysisNotFoundException;
import com.sas.hls.drug.client.DrugServiceManager;
import com.sas.hls.drug.project.Project;
import com.sas.hls.drug.project.exception.ProjectExistsException;
import com.sas.hls.drug.project.exception.ProjectNotFoundException;
import com.sas.hls.messaging.subscription.SubscribableEvent;
import com.sas.hls.messaging.subscription.Subscription;
import com.sas.hls.messaging.subscription.exception.SubscriptionExistsException;
import com.sas.hls.organization.Organization;
import com.sas.hls.organization.exception.OrganizationNotFoundException;
import com.sas.hls.resource.exception.ResourceDeleteException;
import com.sas.hls.resource.exception.ResourceExistsException;
import com.sas.hls.resource.exception.ResourceNotFoundException;
import com.sas.hls.resource.repository.RepositoryFile;
import com.sas.hls.resource.repository.RepositoryResource;
import com.sas.hls.resource.repository.ResourceCheckinSpecification;
import com.sas.hls.resource.repository.VersionType;
import com.sas.hls.resource.repository.exception.RepositoryResourceCheckinException;
import com.sas.hls.resource.repository.exception.RepositoryResourceCheckoutException;
import com.sas.hls.resource.repository.exception.RepositoryResourceCopyException;
import com.sas.hls.resource.repository.exception.RepositoryResourceExistsException;
import com.sas.hls.resource.repository.exception.RepositoryResourceNotFoundException;
import com.sas.hls.resource.repository.exception.RepositoryResourceUndoCheckoutException;
import com.sas.hls.resource.repository.exception.RepositoryResourceVersioningException;
import com.sas.hls.resource.workspace.WorkspaceFile;
import com.sas.hls.resource.workspace.WorkspaceResource;
import com.sas.hls.resource.workspace.exception.WorkspaceResourceCopyException;
import com.sas.hls.resource.workspace.exception.WorkspaceResourceExistsException;
import com.sas.hls.resource.workspace.exception.WorkspaceResourceNotFoundException;
import com.sas.hls.security.audit.AuditEntriesInfo;
import com.sas.hls.security.audit.AuditEntry;
import com.sas.hls.security.audit.AuditEntryQuery;
import com.sas.hls.security.audit.AuditService;
import com.sas.hls.security.audit.exception.AuditEntrySearchException;
import com.sas.hls.security.authentication.exception.AuthenticationException;
import com.sas.hls.security.principal.Principal;
import com.sas.hls.security.user.User;
import com.sas.hls.security.user.UserDescriptor;
import com.sas.hls.security.user.exception.UserDeleteException;
import com.sas.hls.security.user.exception.UserExistsException;
import com.sas.hls.security.user.exception.UserNotFoundException;
import com.sas.hls.security.user.exception.UserPasswordException;

/**
 * 
 * A simple command utility demonstrating the capabilities of the SAS Drug Development API
 * 
 */
public class SASDrugDevCommand {
	
    protected static final Log logger = LogFactory.getLog(SASDrugDevCommand.class);

	private static final String COMMAND_ARGUMENT_USERID_STRING = "-u";
	private static final String COMMAND_ARGUMENT_PASSWORD_STRING = "-p";
	private static final String COMMAND_ARGUMENT_SERVER_STRING = "-s";
    
	private static final String COMMAND_ARGUMENT_COMMENT_STRING = "-comment";
	private static final String COMMAND_ARGUMENT_ORGANIZATION_STRING = "-org";
	private static final String COMMAND_ARGUMENT_PROJECT_STRING = "-project";
	private static final String COMMAND_ARGUMENT_ANALYSIS_STRING = "-analysis";
	
	private static final String COMMAND_ARGUMENT_AUDIT_QUERY_NAME_STRING = "-n";
	private static final String COMMAND_ARGUMENT_AUDIT_QUERY_FROM_DATE_STRING = "-from";
	private static final String COMMAND_ARGUMENT_AUDIT_QUERY_TO_DATE_STRING = "-to";
	private static final String COMMAND_ARGUMENT_AUDIT_QUERY_LOCATION_STRING = "-loc";
	private static final String COMMAND_ARGUMENT_AUDIT_QUERY_OBJECT_TYPE_STRING = "-type";
    private static final String COMMAND_ARGUMENT_AUDIT_QUERY_USERID_STRING = "-userid";
	private static final String DATE_FORMAT = "ddMMMyyyy";
	
	private static final String COMMAND_ARGUMENT_SUBSCRIPTION_PRINCIPAL = "-principal";
	private static final String COMMAND_ARGUMENT_SUBSCRIPTION_EVENT = "-event";
	private static final String COMMAND_ARGUMENT_SUBSCRIPTION_CASCADE = "-cascade";

	private static final String COMMAND_ARGUMENT_STARTING_VERSION_STRING = "-startingVersion";
	private static final String COMMAND_ARGUMENT_CREATE_PARENT_CONTAINERS_STRING = "-createParentContainers";
	private static final String COMMAND_ARGUMENT_ENABLE_VERSIONING_FOR_NEW_FILES_STRING = "-enableVersioningForNewFiles";

	private static final String COMMAND_ARGUMENT_VERSION_TYPE_STRING = "-versionType";
	private static final String COMMAND_ARGUMENT_CHECKOUT_IN_PLACE_STRING = "-checkoutInPlace";
	private static final String COMMAND_ARGUMENT_CUSTOM_VERSION_STRING = "-customVersion";
	private static final String COMMAND_ARGUMENT_VERBOSE_STRING = "-verbose";
	private static final String COMMAND_ARGUMENT_SHOW_CHILDREN_STRING = "-showChildren";
	private static final String COMMAND_ARGUMENT_STATE_STRING = "-state";
	
	private static final String COMMAND_ARGUMENT_USERS_PASSWORD_STRING = "-userPassword";
	private static final String COMMAND_ARGUMENT_DISPLAYNAME_STRING = "-displayName";
	private static final String COMMAND_ARGUMENT_EMAIL_STRING = "-email";

	private static final String COMMAND_ARGUMENT_VERSION_LABEL_STRING = "-versionLabel";
    
    private static final int 	   COMMAND_TEST = 0;
    private static final String COMMAND_TEST_STRING = "-test";
    private static final int 	   COMMAND_REPOSITORY_LIST = 1;
    private static final String COMMAND_LIST_STRING = "-repository_list";    
    private static final int 	   COMMAND_HELP = 2;
    private static final String COMMAND_HELP_STRING = "-help";

    private static final int 	   COMMAND_REPOSITORY_RESOURCE_EXISTS = 6;
    private static final String COMMAND_REPOSITORY_RESOURCE_EXISTS_STRING = "-repository_resource_exists";
    private static final int 	   COMMAND_REPOSITORY_GET_VERSIONS = 7;
    private static final String COMMAND_REPOSITORY_GET_VERSIONS_STRING = "-get_versions";
    private static final int 	   COMMAND_CREATE_REPOSITORY_FOLDER = 9;
    private static final String COMMAND_CREATE_REPOSITORY_FOLDER_STRING = "-create_repository_folder";
    private static final int 	   COMMAND_REPOSITORY_RESOURCE_DELETE = 10;
    private static final String COMMAND_REPOSITORY_RESOURCE_DELETE_STRING = "-delete_from_repository";
	private static final int 	   COMMAND_REPOSITORY_CHECKIN_FROM_WORKSPACE = 11;
    private static final String COMMAND_REPOSITORY_CHECKIN_FROM_WORKSPACE_STRING = "-checkin_file_from_workspace";
	private static final int 	   COMMAND_REPOSITORY_CHECKOUT_TO_WORKSPACE = 12;
    private static final String COMMAND_REPOSITORY_CHECKOUT_TO_WORKSPACE_STRING = "-checkout_file_to_workspace";
	private static final int 	   COMMAND_REPOSITORY_UNDO_CHECKOUT = 15;
    private static final String COMMAND_REPOSITORY_UNDO_CHECKOUT_STRING = "-undo_checkout";
    
    private static final int 	   COMMAND_REPOSITORY_ENABLE_VERSIONING = 13;
    private static final String COMMAND_REPOSITORY_ENABLE_VERSIONING_STRING = "-enable_versioning";
    private static final int 	   COMMAND_REPOSITORY_DISABLE_VERSIONING = 14;
	private static final String COMMAND_REPOSITORY_DISABLE_VERSIONING_STRING = "-disable_versioning";
	private static final int 	   COMMAND_REPOSITORY_CREATE_FILE = 16;
	private static final String COMMAND_REPOSITORY_CREATE_FILE_STRING = "-create_repository_file";
	
	private static final int 	   COMMAND_REPOSITORY_CREATE_VERSIONED_FILE = 18;
	private static final String COMMAND_REPOSITORY_CREATE_VERSIONED_FILE_STRING = "-create_versioned_file";
    
    private static final int 	   COMMAND_REPOSITORY_DOWNLOAD_CONTENT = 8;
    private static final String COMMAND_REPOSITORY_DOWNLOAD_CONTENT_STRING = "-download_repository_file";
	private static final int 	   COMMAND_REPOSITORY_UPDATE_CONTENT = 19;
	private static final String COMMAND_REPOSITORY_UPDATE_CONTENT_STRING = "-update_repository_file";
	private static final int 	   COMMAND_REPOSITORY_UPDATE_VERSIONED_CONTENT = 20;
	private static final String COMMAND_REPOSITORY_UPDATE_VERSIONED_CONTENT_STRING = "-update_versioned_file";

	private static final int 	   COMMAND_REPOSITORY_SAVE_AND_CHECKIN_CONTENT = 22;
	private static final String COMMAND_REPOSITORY_SAVE_AND_CHECKIN_CONTENT_STRING = "-save_content_and_checkin_file";
	private static final int 	   COMMAND_REPOSITORY_SAVE_AND_CHECKIN_VERSIONED_CONTENT = 23;
	private static final String COMMAND_REPOSITORY_SAVE_AND_CHECKIN_VERSIONED_CONTENT_STRING = "-save_content_and_checkin_versioned_file";
	private static final int 	   COMMAND_REPOSITORY_CHECKOUT_AND_GET_CONTENT = 24;
	private static final String COMMAND_REPOSITORY_CHECKOUT_AND_GET_CONTENT_STRING = "-checkout_file_and_get_content";
	private static final int 	   COMMAND_REPOSITORY_GET_SPECIFIED_VERSION_CONTENT = 25;
	private static final String COMMAND_REPOSITORY_GET_SPECIFIED_VERSION_CONTENT_STRING = "-download_file_version";
	private static final int 	   COMMAND_GET_REPOSITORY_FILE_TO_WORKSPACE = 30;
	private static final String COMMAND_GET_REPOSITORY_FILE_TO_WORKSPACE_STRING = "-get_repository_file_to_workspace";
	private static final int 	   COMMAND_REPOSITORY_COPY = 31;
	private static final String COMMAND_REPOSITORY_COPY_STRING = "-repository_copy";
	
    private static final int 	   COMMAND_CREATE_WORKSPACE_FOLDER = 101;
    private static final String COMMAND_CREATE_WORKSPACE_FOLDER_STRING = "-create_workspace_folder";
    private static final int 	   COMMAND_WORKSPACE_DELETE = 102;
    private static final String COMMAND_WORKSPACE_DELETE_STRING = "-delete_from_workspace";
	private static final int 	   COMMAND_WORKSPACE_CREATE_FILE = 103;
	private static final String COMMAND_WORKSPACE_CREATE_FILE_STRING = "-create_workspace_file";
    private static final int 	   COMMAND_WORKSPACE_DOWNLOAD_FILE = 104;
    private static final String COMMAND_WORKSPACE_DOWNLOAD_FILE_STRING = "-download_workspace_file";
    private static final int 	   COMMAND_WORKSPACE_LIST = 105;
    private static final String COMMAND_WORKSPACE_LIST_STRING = "-workspace_list";    
    private static final int 	   COMMAND_WORKSPACE_RESOURCE_EXISTS = 106;
    private static final String COMMAND_WORKSPACE_RESOURCE_EXISTS_STRING = "-workspace_resource_exists";
    private static final int 	   COMMAND_WORKSPACE_UPDATE_FILE = 107;
	private static final String COMMAND_WORKSPACE_UPDATE_FILE_STRING = "-update_workspace_file";
    private static final int 	   COMMAND_WORKSPACE_COPY = 108;
	private static final String COMMAND_WORKSPACE_COPY_STRING = "-workspace_copy";	
	
	private static final int 	COMMAND_PROJECTS_GET_BY_ORG = 227;
	private static final String COMMAND_PROJECTS_GET_BY_ORG_STRING = "-projects_get_by_org";
	private static final int 	COMMAND_PROJECT_EXISTS = 228;
	private static final String COMMAND_PROJECT_EXISTS_STRING = "-project_exists";
	private static final int 	COMMAND_PROJECT_CREATE =229;
	private static final String COMMAND_PROJECT_CREATE_STRING = "-project_create";
	private static final int 	COMMAND_PROJECT_UPDATE = 230;
	private static final String COMMAND_PROJECT_UPDATE_STRING = "-project_update";
	private static final int 	COMMAND_PROJECT_DELETE = 232;
	private static final String COMMAND_PROJECT_DELETE_STRING = "-project_delete";
	private static final int 	COMMAND_PROJECT_GET_BY_ID = 233;
	private static final String COMMAND_PROJECT_GET_BY_ID_STRING = "-project_get_by_id";
	private static final int 	COMMAND_PROJECT_GET_BY_NAME = 234;
	private static final String COMMAND_PROJECT_GET_BY_NAME_STRING = "-project_get_by_name";
	private static final int 	COMMAND_PROJECT_SET_STATE = 235;
	private static final String COMMAND_PROJECT_SET_STATE_STRING = "-project_set_state";
	
	private static final int    COMMAND_ANALYSES_GET_BY_PROJECT = 340;
	private static final String COMMAND_ANALYSES_GET_BY_PROJECT_STRING = "-analyses_get_by_project";
	private static final int    COMMAND_ANALYSIS_EXISTS = 341;
	private static final String COMMAND_ANALYSIS_EXISTS_STRING = "-analysis_exists";
	private static final int    COMMAND_ANALYSIS_CREATE = 342;
	private static final String COMMAND_ANALYSIS_CREATE_STRING = "-analysis_create";
	private static final int    COMMAND_ANALYSIS_UPDATE = 343;
	private static final String COMMAND_ANALYSIS_UPDATE_STRING = "-analysis_update";
	private static final int    COMMAND_ANALYSIS_DELETE = 345;
	private static final String COMMAND_ANALYSIS_DELETE_STRING = "-analysis_delete";
	private static final int    COMMAND_ANALYSIS_GET_BY_ID = 346;
	private static final String COMMAND_ANALYSIS_GET_BY_ID_STRING = "-analysis_get_by_id";
	private static final int    COMMAND_ANALYSIS_GET_BY_NAME = 347;
	private static final String COMMAND_ANALYSIS_GET_BY_NAME_STRING = "-analysis_get_by_name";
	private static final int    COMMAND_ANALYSIS_SET_STATE = 348;
	private static final String COMMAND_ANALYSIS_SET_STATE_STRING = "-analysis_set_state";
		
    private static final int    COMMAND_USER_CREATE = 401;
	private static final String COMMAND_USER_CREATE_STRING = "-user_create";
	private static final int    COMMAND_USER_EXISTS = 402;
	private static final String COMMAND_USER_EXISTS_STRING = "-user_exists";
	private static final int    COMMAND_USER_DELETE = 403;
	private static final String COMMAND_USER_DELETE_STRING = "-user_delete";
	private static final int    COMMAND_USER_GET = 404;
	private static final String COMMAND_USER_GET_STRING = "-user_get";
	private static final int    COMMAND_USER_RESET_PASSWORD = 405;
	private static final String COMMAND_USER_RESET_PASSWORD_STRING = "-user_reset";
	
	private static final int    COMMAND_AUDIT_QUERY = 501;
	private static final String COMMAND_AUDIT_QUERY_STRING = "-exportAudit";
	
	private static final int    COMMAND_SUBSCRIPTION_GET_EVENTS = 601;
    private static final String COMMAND_SUBSCRIPTION_GET_EVENTS_STRING = "-getSubscriptionEvents";
    private static final int    COMMAND_SUBSCRIPTION_CREATE = 602;
    private static final String COMMAND_SUBSCRIPTION_CREATE_STRING = "-createSubscriptionForUser";
    private static final int    COMMAND_SUBSCRIPTION_GET_BY_USER = 603;
    private static final String COMMAND_SUBSCRIPTION_GET_BY_USER_STRING = "-getSubscriptionsByUser";
    private static final int    COMMAND_SUBSCRIPTION_GET_BY_PATH_AND_EVENT = 604;
    private static final String COMMAND_SUBSCRIPTION_GET_BY_PATH_AND_EVENT_STRING = "-getSubscriptionsByPathAndEvent";
		
	/**
	 * Simple command-line sample program using the SAS Drug Development API.
	 * 
	 * @param argv 
	 */
	public static void main(String[] argv) {		

		URL serverURL = null;
		String serverURLString = null;
		
		String userid = null;
		char[] password = null;
		
		/* Options common across multiple commands */
		String workspaceOrRepositoryPath = null;
		String localFilePath = null;
		String versionLabel = "";
		boolean createParentContainers = true;
		String comment = "";
		String startingVersion = "1.0";
		boolean enableVersioningForNewFiles = false;
		String versionType = "MINOR";
		boolean checkoutInPlace = false;
		String customVersion = null;
		boolean verboseMode = false;
		boolean showChildren = false;
		String sourcePath = null;
		String targetPath = null;
		String orgName = "";
		String projectName = "";
		String analysisName = "";
		String stateValue = "";
		String usersDisplayName = "";
		String usersId = "";
		String usersEmail = "";
		String usersPassword = "";
		String name = null;
		String location = null;
		String type = null;
		String fromDate = null;
		String toDate = null;
		String eventName = null;
		String principal = null;
		String user = null;
		boolean cascade = false;
		int command = -1;
		
		logger.info("SASDrugDevCommand begins.");
		String cmd = "";
		for(String arg:argv){
			cmd+=(arg+" ");
		}
		logger.info("Called with arguments: "+cmd);
                 
		if (argv.length < 1) {
			showUsage();
			System.exit(1);
		}

		//Parsing of the command line arguments
		for (int i = 0; i < argv.length; i++) {
			
			/* arguments common to all commands */
			if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_SERVER_STRING)) {
				i++;
				if (i < argv.length) {
					serverURLString = argv[i];					
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_USERID_STRING)) {
				i++;
				if (i < argv.length) userid = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_PASSWORD_STRING)) {
				i++;
				if (i < argv.length) password = argv[i].toCharArray();
				
				/* arguments common to several commands */	
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_COMMENT_STRING)) {
				i++;
				if (i < argv.length) comment = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_STATE_STRING)) {
				i++;
				if (i < argv.length) stateValue = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_ORGANIZATION_STRING)) {
				i++;
				if (i < argv.length) orgName = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_PROJECT_STRING)) {
				i++;
				if (i < argv.length) projectName = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_ANALYSIS_STRING)) {
				i++;
				if (i < argv.length) analysisName = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_STARTING_VERSION_STRING)) {
				i++;
				if (i < argv.length) startingVersion = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_CREATE_PARENT_CONTAINERS_STRING)) {
				i++;
				if (i < argv.length) createParentContainers = argv[i].equalsIgnoreCase("true");
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_ENABLE_VERSIONING_FOR_NEW_FILES_STRING)) {
				i++;
				if (i < argv.length) enableVersioningForNewFiles = argv[i].equalsIgnoreCase("true");
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_VERSION_TYPE_STRING)) {
				i++;
				if (i < argv.length) versionType = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_CHECKOUT_IN_PLACE_STRING)) {
				i++;
				if (i < argv.length) checkoutInPlace = argv[i].equalsIgnoreCase("true");
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_CUSTOM_VERSION_STRING)) {
				i++;
				if (i < argv.length) customVersion = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_VERBOSE_STRING)) {
                verboseMode = true;				
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_SHOW_CHILDREN_STRING)) {
				showChildren = true;				
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_VERSION_LABEL_STRING)) {
				i++;
				if (i < argv.length) versionLabel = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_USERS_PASSWORD_STRING)) {
				i++;
				if (i < argv.length) usersPassword = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_DISPLAYNAME_STRING)) {
				i++;
				if (i < argv.length) usersDisplayName = argv[i];
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_EMAIL_STRING)) {
				i++;
				if (i < argv.length) usersEmail = argv[i];

			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_NAME_STRING)) {
                i++;
                if (i < argv.length) name = argv[i];	
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_LOCATION_STRING)) {
                i++;
                if (i < argv.length) location = argv[i];    
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_OBJECT_TYPE_STRING)) {
                i++;
                if (i < argv.length) type = argv[i];    	
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_TO_DATE_STRING)) {
                i++;
                if (i < argv.length) toDate = argv[i];       
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_FROM_DATE_STRING)) {
                i++;
                if (i < argv.length) fromDate = argv[i];       
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_USERID_STRING)) {
                i++;
                if (i < argv.length) user = argv[i];       
			} else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_AUDIT_QUERY_USERID_STRING)) {
	                i++;
	                if (i < argv.length) user = argv[i];       
            } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_SUBSCRIPTION_CASCADE)) {
                cascade = true;
            } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_SUBSCRIPTION_PRINCIPAL)) {
                i++;
                if (i < argv.length) principal = argv[i];  
            } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_SUBSCRIPTION_EVENT)) {
                i++;
                if (i < argv.length) eventName = argv[i];                
                
				/* commands */
			} else if (argv[i].equalsIgnoreCase(COMMAND_TEST_STRING)) {
				command = COMMAND_TEST;
			} else if (argv[i].equalsIgnoreCase(COMMAND_HELP_STRING)) {
				command = COMMAND_HELP;
			} else if (argv[i].equalsIgnoreCase(COMMAND_LIST_STRING)) {
				command = COMMAND_REPOSITORY_LIST;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				} 
			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_LIST_STRING)) {
				command = COMMAND_WORKSPACE_LIST;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_RESOURCE_EXISTS_STRING)) {
				command = COMMAND_REPOSITORY_RESOURCE_EXISTS;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_GET_VERSIONS_STRING)) {
				command = COMMAND_REPOSITORY_GET_VERSIONS;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_GET_REPOSITORY_FILE_TO_WORKSPACE_STRING)) {
				command = COMMAND_GET_REPOSITORY_FILE_TO_WORKSPACE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_CREATE_REPOSITORY_FOLDER_STRING)) {
				command = COMMAND_CREATE_REPOSITORY_FOLDER;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_RESOURCE_DELETE_STRING)) {
				command = COMMAND_REPOSITORY_RESOURCE_DELETE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_CHECKIN_FROM_WORKSPACE_STRING)) {
				command = COMMAND_REPOSITORY_CHECKIN_FROM_WORKSPACE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_CHECKOUT_TO_WORKSPACE_STRING)) {
				command = COMMAND_REPOSITORY_CHECKOUT_TO_WORKSPACE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_ENABLE_VERSIONING_STRING)) {
				command = COMMAND_REPOSITORY_ENABLE_VERSIONING;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_DISABLE_VERSIONING_STRING)) {
				command = COMMAND_REPOSITORY_DISABLE_VERSIONING;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_UNDO_CHECKOUT_STRING)) {
				command = COMMAND_REPOSITORY_UNDO_CHECKOUT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_DOWNLOAD_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_DOWNLOAD_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to write it to
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_UPDATE_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_UPDATE_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file with the new content
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_UPDATE_VERSIONED_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_UPDATE_VERSIONED_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file with the new content
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_COPY_STRING)) {
				command = COMMAND_REPOSITORY_COPY;
				i++;
				if (i < argv.length) {
					sourcePath = argv[i];
					i++;
					if (i < argv.length) {
						targetPath = argv[i];
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_COPY_STRING)) {
				command = COMMAND_WORKSPACE_COPY;
				i++;
				if (i < argv.length) {
					sourcePath = argv[i];
					i++;
					if (i < argv.length) {
						targetPath = argv[i];
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_CREATE_FILE_STRING)) {
				command = COMMAND_WORKSPACE_CREATE_FILE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file with the new content
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_DOWNLOAD_FILE_STRING)) {
				command = COMMAND_WORKSPACE_DOWNLOAD_FILE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to write it to
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_RESOURCE_EXISTS_STRING)) {
				command = COMMAND_WORKSPACE_RESOURCE_EXISTS;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_CREATE_FILE_STRING)) {
				command = COMMAND_REPOSITORY_CREATE_FILE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_CREATE_VERSIONED_FILE_STRING)) {
				command = COMMAND_REPOSITORY_CREATE_VERSIONED_FILE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_UPDATE_FILE_STRING)) {
				command = COMMAND_WORKSPACE_UPDATE_FILE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_SAVE_AND_CHECKIN_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_SAVE_AND_CHECKIN_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_SAVE_AND_CHECKIN_VERSIONED_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_SAVE_AND_CHECKIN_VERSIONED_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_CHECKOUT_AND_GET_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_CHECKOUT_AND_GET_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_REPOSITORY_GET_SPECIFIED_VERSION_CONTENT_STRING)) {
				command = COMMAND_REPOSITORY_GET_SPECIFIED_VERSION_CONTENT;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
					i++;
					if (i < argv.length) {
						localFilePath = argv[i]; //the file to get contents from
					}
				}

			} else if (argv[i].equalsIgnoreCase(COMMAND_CREATE_WORKSPACE_FOLDER_STRING)) {
				command = COMMAND_CREATE_WORKSPACE_FOLDER;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}

			} else if (argv[i].equalsIgnoreCase(COMMAND_WORKSPACE_DELETE_STRING)) {
				command = COMMAND_WORKSPACE_DELETE;
				i++;
				if (i < argv.length) {
					workspaceOrRepositoryPath = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_EXISTS_STRING)) {
				command = COMMAND_ANALYSIS_EXISTS;
				i++;
				if (i < argv.length) {
					analysisName = argv[i]; //have them specify a -project option rather than have a chain of args here
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSES_GET_BY_PROJECT_STRING)) {
				command = COMMAND_ANALYSES_GET_BY_PROJECT;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_CREATE_STRING)) {
				command = COMMAND_ANALYSIS_CREATE;
				i++;
				if (i < argv.length) {
					analysisName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_UPDATE_STRING)) {
				command = COMMAND_ANALYSIS_UPDATE;
				i++;
				if (i < argv.length) {
					analysisName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_DELETE_STRING)) {
				command = COMMAND_ANALYSIS_DELETE;
				i++;
				if (i < argv.length) {
					analysisName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_GET_BY_ID_STRING)) {
				command = COMMAND_ANALYSIS_GET_BY_ID;
				i++;
				if (i < argv.length) {
					analysisName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_GET_BY_NAME_STRING)) {
				command = COMMAND_ANALYSIS_GET_BY_NAME;
				i++;
				if (i < argv.length) {
					analysisName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_ANALYSIS_SET_STATE_STRING)) {
				command = COMMAND_ANALYSIS_SET_STATE;
				i++;
				if (i < argv.length) {
					analysisName = argv[i];
					i++;
					if (i < argv.length) {
						stateValue = argv[i]; 
					}
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECTS_GET_BY_ORG_STRING)) {
				command = COMMAND_PROJECTS_GET_BY_ORG;
				i++;
				if (i < argv.length) {
					orgName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_EXISTS_STRING)) {
				command = COMMAND_PROJECT_EXISTS;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_CREATE_STRING)) {
				command = COMMAND_PROJECT_CREATE;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_UPDATE_STRING)) {
				command = COMMAND_PROJECT_UPDATE;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_DELETE_STRING)) {
				command = COMMAND_PROJECT_DELETE;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_GET_BY_ID_STRING)) {
				command = COMMAND_PROJECT_GET_BY_ID;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_GET_BY_NAME_STRING)) {
				command = COMMAND_PROJECT_GET_BY_NAME;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_PROJECT_SET_STATE_STRING)) {
				command = COMMAND_PROJECT_SET_STATE;
				i++;
				if (i < argv.length) {
					projectName = argv[i];
					i++;
					if (i < argv.length) {
						stateValue = argv[i]; //the state to set the project to
					}
				}
			
			} else if (argv[i].equalsIgnoreCase(COMMAND_USER_CREATE_STRING)) {
				command = COMMAND_USER_CREATE;
				i++;
				if (i < argv.length) {
					usersId = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_USER_EXISTS_STRING)) {
				command = COMMAND_USER_EXISTS;
				i++;
				if (i < argv.length) {
					usersId = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_USER_DELETE_STRING)) {
				command = COMMAND_USER_DELETE;
				i++;
				if (i < argv.length) {
					usersId = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_USER_GET_STRING)) {
				command = COMMAND_USER_GET;
				i++;
				if (i < argv.length) {
					usersId = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase(COMMAND_USER_RESET_PASSWORD_STRING)) {
				command = COMMAND_USER_RESET_PASSWORD;
				i++;
				if (i < argv.length) {
					usersId = argv[i];
				}
            } else if (argv[i].equalsIgnoreCase(COMMAND_AUDIT_QUERY_STRING)) {
                command = COMMAND_AUDIT_QUERY;
                i++;
                if (i < argv.length) {
                    localFilePath = argv[i];
                }
            } else if (argv[i].equalsIgnoreCase(COMMAND_SUBSCRIPTION_GET_EVENTS_STRING)) {
                command = COMMAND_SUBSCRIPTION_GET_EVENTS;
                i++;
                if (i < argv.length) {
                    type = argv[i];
                }
            } else if (argv[i].equalsIgnoreCase(COMMAND_SUBSCRIPTION_CREATE_STRING)) {
                command = COMMAND_SUBSCRIPTION_CREATE;
                i++;
                if (i < argv.length) {
                    workspaceOrRepositoryPath  = argv[i];
                }            
                
            } else if (argv[i].equalsIgnoreCase(COMMAND_SUBSCRIPTION_GET_BY_USER_STRING)) {
                command = COMMAND_SUBSCRIPTION_GET_BY_USER;
                i++;
                if (i < argv.length) {
                    principal  = argv[i];
                }          
            } else if (argv[i].equalsIgnoreCase(COMMAND_SUBSCRIPTION_GET_BY_PATH_AND_EVENT_STRING)) {
                command = COMMAND_SUBSCRIPTION_GET_BY_PATH_AND_EVENT;
                i++;
                if (i < argv.length) {
                    workspaceOrRepositoryPath  = argv[i];
                }            
                              
			} else {
				displayErrorMessage("unrecognized parameter: "+argv[i]);				
			}
		}
		
		
        
		//if they just want the help command, we don't need other parms so let's check for that
		if(command == COMMAND_HELP){
		    showUsage();
		    System.exit(0);
		}
		
		//You must have told me you want to do SOMETHING....
		if (command < 0) {
			recordError("No valid command specified.");
			showUsage();
			System.exit(1);
		}
		
		//let's verify we have enough information to make a connection and run all the other commands
	
		//if serverURL not passed in arg list then we need to prompt for it.
		if (serverURLString == null) {
			serverURLString = String.valueOf(readInput("Server URL: "));
		}
		
		//validate the URL value
		try {
			serverURL = new URL(serverURLString);
		} catch (MalformedURLException me) {
			recordError("The server URL is not valid.",me);
			System.exit(1);
		}
	
		//get credential information if we didn't already get it from arguments
		if (userid == null || userid.length() == 0) {
			userid = String.valueOf(readInput("Username: "));
		}
		if (password == null || String.valueOf(password).length() == 0) {
			password = readInput("Password: ");
		}
		
		//now create our session
		Session session = null;
		boolean connectionSuccessful = true;
		try{
			try {
				//create a new authenticated session with SDD
				session = SessionFactory.newSession(serverURL, userid, String.valueOf(password));
			} catch (AuthenticationException ex) {
				// wrong uid/password, or account is locked etc...
				recordError(ex.getLocalizedMessage(),ex);
				connectionSuccessful = false;
			} catch (HttpException ex) {
				// failure in communicating with server etc...
				recordError("Could not create session: "+ex.getLocalizedMessage(),ex);
				connectionSuccessful = false;
			} catch (ConfigurationException e){
				//probably a mismatch between client and server versions of the API
				recordError("Configuration error: "+e.getLocalizedMessage());
				connectionSuccessful = false;
			}
			
			
			
		if (command == COMMAND_TEST){
			if (connectionSuccessful) {
				System.out.println("Connection successful!");
			} else {
				recordError("\n**Connection to " + serverURL + " failed.**\n");
			}			
		} else if (session == null) {
			displayNoSessionErrorMessage();
		} else switch (command) {
									
				case COMMAND_REPOSITORY_LIST: 
					doRepositoryListCommand(session, workspaceOrRepositoryPath, verboseMode, showChildren);
					break;
					
				case COMMAND_WORKSPACE_LIST: 
					doWorkspaceListCommand(session, workspaceOrRepositoryPath, verboseMode, showChildren);
					break;					

				case COMMAND_REPOSITORY_RESOURCE_EXISTS: 
					doRepositoryResourceExistsCommand(session, workspaceOrRepositoryPath);
					break;
					
				case COMMAND_WORKSPACE_RESOURCE_EXISTS: 
					doWorkspaceResourceExistsCommand(session, workspaceOrRepositoryPath);
					break;
										
				case COMMAND_REPOSITORY_GET_VERSIONS: 
					doResourceGetVersionsCommand(session, workspaceOrRepositoryPath, verboseMode);
					break;
					
				case COMMAND_REPOSITORY_DOWNLOAD_CONTENT: 
					doRepositoryDownloadContentCommand(session, workspaceOrRepositoryPath, localFilePath);
					break;	
					
					case COMMAND_WORKSPACE_DOWNLOAD_FILE: 
					doWorkspaceDownloadFileCommand(session, workspaceOrRepositoryPath, localFilePath);
					break;			
					
					case COMMAND_GET_REPOSITORY_FILE_TO_WORKSPACE: 
					doGetFileToWorkspaceCommand(session, workspaceOrRepositoryPath, versionLabel);
					break;			
					
				case COMMAND_REPOSITORY_UPDATE_CONTENT: 
					doRepositoryUpdateContentCommand(session, workspaceOrRepositoryPath, localFilePath, comment);
					break;	

				case COMMAND_REPOSITORY_UPDATE_VERSIONED_CONTENT: 
					doRepositoryUpdateVersionedContentCommand(session, workspaceOrRepositoryPath, localFilePath,enableVersioningForNewFiles, versionType, customVersion,  comment);
					break;	

				case COMMAND_REPOSITORY_CREATE_FILE: 
					doRepositoryCreateFileCommand(session, workspaceOrRepositoryPath, localFilePath, comment, createParentContainers);
					break;	
					
					case COMMAND_WORKSPACE_CREATE_FILE: 
					doWorkspaceCreateFileCommand(session, workspaceOrRepositoryPath, localFilePath, createParentContainers);
					break;	
					
				case COMMAND_REPOSITORY_CREATE_VERSIONED_FILE: 
					doCreateVersionedFileCommand(session, workspaceOrRepositoryPath, localFilePath, startingVersion, comment, createParentContainers);
					break;	
					
				case COMMAND_CREATE_REPOSITORY_FOLDER: 
					doCreateRepositoryFolderCommand(session, workspaceOrRepositoryPath, createParentContainers);
					break;	
					
				case COMMAND_REPOSITORY_COPY: 
					doRepositoryCopyCommand(session, sourcePath, targetPath);
					break;	
					
				case COMMAND_WORKSPACE_COPY: 
					doWorkspaceCopyCommand(session, sourcePath, targetPath);
					break;	
					
				case COMMAND_CREATE_WORKSPACE_FOLDER: 
					doCreateWorkspaceFolderCommand(session, workspaceOrRepositoryPath, createParentContainers);
					break;	
					
				case COMMAND_REPOSITORY_RESOURCE_DELETE: 
					doRepositoryDeleteCommand(session, workspaceOrRepositoryPath);
					break;	
					
				case COMMAND_WORKSPACE_DELETE: 
					doWorkspaceDeleteCommand(session, workspaceOrRepositoryPath);
					break;	
					
				case COMMAND_REPOSITORY_CHECKOUT_TO_WORKSPACE: 
					doCheckOutToWorkspaceCommand(session,  workspaceOrRepositoryPath);
					break;	
					
				case COMMAND_REPOSITORY_UNDO_CHECKOUT: 
					doUndoCheckOutCommand(session,  workspaceOrRepositoryPath);
					break;	
					
				case COMMAND_REPOSITORY_DISABLE_VERSIONING: 
					doDisableVersioningCommand(session, workspaceOrRepositoryPath, comment);
					break;	
					
				case COMMAND_REPOSITORY_CHECKIN_FROM_WORKSPACE: 
					doCheckinFromWorkspaceCommand(session,  workspaceOrRepositoryPath, enableVersioningForNewFiles, versionType, customVersion, checkoutInPlace, comment);
					break;	
					
				case COMMAND_REPOSITORY_ENABLE_VERSIONING: 
					doEnableVersioningCommand(session, workspaceOrRepositoryPath, comment, startingVersion);
					break;	

				case COMMAND_REPOSITORY_SAVE_AND_CHECKIN_VERSIONED_CONTENT: 
					doSaveAndCheckinVersionedContentCommand(session,  workspaceOrRepositoryPath, localFilePath, enableVersioningForNewFiles, versionType, customVersion, comment);
					break;	
					
				case COMMAND_WORKSPACE_UPDATE_FILE: 
					doWorkspaceUploadFileCommand(session,  workspaceOrRepositoryPath, localFilePath);
					break;	
					
				case COMMAND_REPOSITORY_SAVE_AND_CHECKIN_CONTENT: 
					doSaveAndCheckinContentCommand(session,  workspaceOrRepositoryPath, localFilePath,comment);
					break;	
					
				case COMMAND_REPOSITORY_CHECKOUT_AND_GET_CONTENT: 
					doCheckoutAndGetContentCommand(session,  workspaceOrRepositoryPath, localFilePath);
					break;	
					
				case COMMAND_REPOSITORY_GET_SPECIFIED_VERSION_CONTENT: 
					doGetSpecifiedVersionContentCommand(session,  workspaceOrRepositoryPath, versionLabel, localFilePath);
					break;	
					
				case COMMAND_PROJECTS_GET_BY_ORG: 
					doDisplayProjectsByOrg(session, userid, orgName);
					break;	
					
				case COMMAND_PROJECT_EXISTS: 
					doProjectExistsCommand(session, userid, projectName, orgName);
					break;	
					
				case COMMAND_PROJECT_GET_BY_ID: 
					doProjectGetByIDCommand(session, userid, projectName);
					break;	
					
				case COMMAND_PROJECT_GET_BY_NAME: 
					doProjectGetByNameCommand(session, userid, projectName, orgName);
					break;	
					
				case COMMAND_PROJECT_DELETE: 
					doProjectDeleteCommand(session, userid, projectName, orgName);
					break;	
										
				case COMMAND_PROJECT_CREATE: 
					doProjectCreateCommand(session, userid, projectName, orgName);
					break;						
					
				case COMMAND_ANALYSIS_EXISTS: 
					doAnalysisExistsCommand(session, userid, analysisName, projectName, orgName);
					break;	
					
					
				case COMMAND_ANALYSES_GET_BY_PROJECT: 
					doDisplayAnalysesByProject(session, userid, projectName, orgName);
					break;	
					
				case COMMAND_ANALYSIS_GET_BY_ID: 
					doAnalysisGetByIDCommand(session, userid, analysisName);
					break;	
					
				case COMMAND_ANALYSIS_GET_BY_NAME: 
					doAnalysisGetByNameCommand(session, userid, analysisName, projectName, orgName);
					break;	
					
				case COMMAND_ANALYSIS_DELETE: 
					doAnalysisDeleteCommand(session, userid, analysisName, projectName, orgName);
					break;						
					
				case COMMAND_ANALYSIS_CREATE: 
					doAnalysisCreateCommand(session, userid, analysisName, projectName, stateValue, orgName, comment);
					break;	
					
				case COMMAND_USER_CREATE: 
					doUserCreateCommand(session, usersId, usersDisplayName, usersEmail,usersPassword);
					break;	
					
				case COMMAND_USER_EXISTS: 
					doUserExistsCommand(session, usersId);
					break;	
					
				case COMMAND_USER_DELETE: 
					doUserDeleteCommand(session, usersId);
					break;	
					
				case COMMAND_USER_GET: 
					doUserGetCommand(session, usersId);
					break;	
					
				case COMMAND_USER_RESET_PASSWORD: 
					doUserResetPasswordCommand(session, usersId,usersPassword);
					break;	
					
				case COMMAND_AUDIT_QUERY:
                    doAuditQuery(session, user, name, location, type, fromDate, toDate, localFilePath);
                    break;  	
 
				case COMMAND_SUBSCRIPTION_GET_EVENTS:
				    doGetSubscriptionEvents(session, type);
                    break;      
                    
				case COMMAND_SUBSCRIPTION_CREATE:
				    doSubscriptionCreate(session, workspaceOrRepositoryPath, eventName, user, cascade);
                    break;      
                    
                case COMMAND_SUBSCRIPTION_GET_BY_USER:
                    doGetSubscriptionsByUser(session, principal);
                    break;      
                    
                case COMMAND_SUBSCRIPTION_GET_BY_PATH_AND_EVENT:
                    doGetSubscriptionsByPathAndEvent(session, workspaceOrRepositoryPath, eventName);
                    break;    
                    
				default:
					showUsage();
					System.exit(1);
					break;
			}
		} catch (Throwable t){
			recordError("Unexpected Error:  "+t.getLocalizedMessage()); //not expected
		} finally {
			if(session != null){
				session.invalidate();//release our connection to the server.
			}			
		}
	}
	
	private static void doGetSubscriptionEvents(Session session, String objectType){
	   Set<SubscribableEvent> events = ((DrugServiceManager) session.getServiceManager()).getSubscriptionService().getSubscribableEvents(objectType);
	   System.out.println("Subscribable events:\n");
	   for (SubscribableEvent event : events){
	       System.out.println(event.getEvent() + " (canCascade: " + event.isCascadable() + ")");
	   }	   
	   displaySuccessMessage();
	}
	
	private static void doSubscriptionCreate(Session session, String path, String event, String userid, boolean cascade){
	    try {
            RepositoryResource resource = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getResource(path);
            Set<SubscribableEvent> events = ((DrugServiceManager) session.getServiceManager()).getSubscriptionService().getSubscribableEvents(resource.getObjectType());
            SubscribableEvent eventToSubscribeTo = null;
            for (SubscribableEvent e : events){
                if (e.getEvent().equalsIgnoreCase(event)){
                    eventToSubscribeTo = e;
                    break;
                }
            }       
            
            if (eventToSubscribeTo == null){
                displayErrorMessage("The event " + event + " is not available at the path given.  Make sure the event exists for that object type.");
            }
            Principal p = null;
            if (userid != null){
                p = ((DrugServiceManager) session.getServiceManager()).getUserService().getUserDescriptorByUserId(userid);
            } 
            
            ((DrugServiceManager) session.getServiceManager()).getSubscriptionService().createSubscription(path, eventToSubscribeTo, p, cascade);
            
            displaySuccessMessage();
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Path not found. " + e.getLocalizedMessage());
        } catch (UserNotFoundException e) {
            displayErrorMessage("User not found. " + e.getLocalizedMessage());
        } catch (SubscriptionExistsException e) {
            displayErrorMessage("Alread exists. " + e.getLocalizedMessage());
        }
    }
	
	private static void doGetSubscriptionsByUser(Session session, String userid){
        try {
            UserDescriptor user = null;
            if (userid != null){
                user = ((DrugServiceManager) session.getServiceManager()).getUserService().getUserDescriptorByUserId(userid);
            } 
            Set<Subscription> subs = ((DrugServiceManager) session.getServiceManager()).getSubscriptionService().getSubscriptionsByPrincipal(user);
       
            if (subs == null || subs.isEmpty()){
                System.out.println("No subscriptions found.");
            }else{
                System.out.println("Subscriptions for " + userid + ":");
                for (Subscription sub : subs){
                    System.out.println("Path: " + sub.getPath() + ", event: " + sub.getSubscriptionEvent().getEvent());
                }  
                displaySuccessMessage();
            }        
        } catch (UserNotFoundException e) {
            displayErrorMessage("User not found. " + e.getLocalizedMessage());
        }  
    }
	
	private static void doGetSubscriptionsByPathAndEvent(Session session, String path, String event){
        try {
            RepositoryResource resource = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getResource(path);
            Set<SubscribableEvent> events = ((DrugServiceManager) session.getServiceManager()).getSubscriptionService().getSubscribableEvents(resource.getObjectType());
            SubscribableEvent eventToSubscribeTo = null;
            for (SubscribableEvent e : events){
                if (e.getEvent().equalsIgnoreCase(event)){
                    eventToSubscribeTo = e;
                    break;
                }
            }       
            Set<Subscription> subs = ((DrugServiceManager) session.getServiceManager()).getSubscriptionService().getSubscriptionsByPathAndEvent(path, eventToSubscribeTo);
       
            if (subs == null || subs.isEmpty()){
                System.out.println("No subscriptions found.");
            }else{
                System.out.println("Subscriptions for " + event + " at " + path + ":");
                for (Subscription sub : subs){
                    System.out.println("Subscriber: " + sub.getPrincipal().getPrincipalId() + " (" + sub.getPrincipal().getObjectType() + ")");
                }  
                displaySuccessMessage();
            }        
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Path not found. " + e.getLocalizedMessage());
        }  
    }


	private static void doAuditQuery(Session session, String user, String name, String location, String type, String fromDate,
            String toDate, String localFilePath) {
	    
	    Date fDate = null;
        Date tDate = null;
        
        try {
            if (toDate != null){
                tDate = new SimpleDateFormat(DATE_FORMAT).parse(toDate);
            }
        } catch (ParseException e) {
            displayErrorMessage("To date is not in the valid format.  Please use format: " + DATE_FORMAT);
        } 
        try {          
            if (fromDate != null){
                fDate = new SimpleDateFormat(DATE_FORMAT).parse(fromDate); 
            }
        } catch (ParseException e) {
            displayErrorMessage("From date is not in the valid format.  Please use format: " + DATE_FORMAT);
        } 
        
        List<String> typesList = null;
        if (type != null){
            String typeArray[] = type.split(",");
            typesList = Arrays.asList(typeArray);           
        }
        
        if (localFilePath == null){
            displayErrorMessage("Local file path must be provided.");
        }
 
        AuditEntryQuery auditQuery = new AuditEntryQuery(user, typesList, name, location, null, fDate, tDate); 
	    AuditService auditService = ((DrugServiceManager) session.getServiceManager()).getAuditService();
	    AuditEntriesInfo auditEntriesInfo = auditService.getAuditEntriesInfoByQuery(auditQuery);
	    File f = new File(localFilePath);
	    FileOutputStream fos = null;
	    int start = 1;
	    int total = auditEntriesInfo.getRowCount();
	    int end = total;
	    int max = auditService.getPageSizeLimit();
	    List<AuditEntry> entries = null;
	    try {
            fos = new FileOutputStream(f);
            if (end > max){
                end = max;
            }
            StringBuffer auditEntryString = new StringBuffer();
            if (total != 0){
                entries = auditService.getAuditEntries(auditEntriesInfo.getAuditToken(), start, end);
                while (end <= total) {
                    if ((start + max) > total){
                        end = total;
                    } else {
                        end = start + max;
                    }
                    entries = auditService.getAuditEntries(auditEntriesInfo.getAuditToken(), start, end);
                    for (AuditEntry entry : entries){
                         auditEntryString.append(entry.getAction() + "|" + entry.getUserId() + "|" 
                                + entry.getSourceType() + "|" + entry.getSourceName() + "|" 
                                + (entry.getSourceLocation() != null ? entry.getSourceLocation() : "n/a") 
                                + "|" + entry.getTimestamp() + "\n");
                        
                    }
                    start = end + 1;
                    if (start > total){
                        break;
                    }
              }
              IOUtils.write(auditEntryString, fos);
              System.out.println("Audit entries written to " + f.getAbsolutePath());
            }else{
                System.out.println("The audit query returned no entries.");
            }
        } catch (AuditEntrySearchException e) {
            displayErrorMessage("AuditEntrySearchException Error:  "+e.getLocalizedMessage());
        } catch (FileNotFoundException e) {
            displayErrorMessage(e.getLocalizedMessage());
        } catch (IOException e) {
            displayErrorMessage(e.getLocalizedMessage());
        } finally {
            auditService.clearAuditEntriesInfo(auditEntriesInfo.getAuditToken());
            if (fos != null){
                try {
                    fos.close();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
	              
    }

    private static void doRepositoryResourceExistsCommand(Session session, String resourcePath){					
		boolean exists = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().exists(resourcePath);
		System.out.println("Resource "+resourcePath+" exists = "+exists);
		displaySuccessMessage();
	}

	private static void doWorkspaceResourceExistsCommand(Session session, String resourcePath){					
		boolean exists = ((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().exists(resourcePath);
		System.out.println("Workspace resource "+resourcePath+" exists = "+exists);
		displaySuccessMessage();
	}

	private static void doResourceGetVersionsCommand(Session session, String resourcePath, boolean verbose){					
		List<RepositoryFile> versions;
		try {
			versions = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getVersions(resourcePath);
			System.out.println("Resource "+resourcePath+" has "+versions.size()+" version(s).");
			for(RepositoryFile version:versions){
				if(!verbose){
					System.out.println("   Version "+version.getVersionLabel()+" was last modified on "+version.getLastModifiedDate());
				} else {	
					System.out.println("    ");
					System.out.println("path: "+version.getPath()); //constant across versions...dumb to repeat.
					System.out.println("   is versioned: "+version.isVersioned());
					System.out.println("   is checkedOut: "+version.isCheckedOut());
					System.out.println("   version: "+version.getVersionLabel());
					System.out.println("   size: "+version.getSize());				
					
					List<Attribute> childAttrs = version.getAttributes();
					for(Attribute attr:childAttrs){
						System.out.println("      Attribute "+attr.getName()+": "+attr.getValue());
					}
				}
			}
			displaySuccessMessage();
		} catch (RepositoryResourceNotFoundException e) {
			displayErrorMessage("Resource Not Found:  "+e.getPath());			
		} catch (RepositoryResourceVersioningException e) {
			displayErrorMessage("File is not versioned:  "+e.getPath());
		}
	}
	
	private static void doRepositoryDownloadContentCommand(Session session, String resourcePath, String outputFilePath) {
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().downloadFile(resourcePath, "", outputFilePath);				
			displaySuccessMessage();
		} catch (RepositoryResourceNotFoundException rnf){
			displayErrorMessage("Resource Not Found:  "+rnf.getPath());			
		} catch (IOException ioe){
			displayErrorMessage("IO Error:  "+ioe.getLocalizedMessage());
		}
	}
	
	private static void doWorkspaceDownloadFileCommand(Session session, String resourcePath, String outputFilePath) {	
		try{
			if(outputFilePath != null){				
				((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().downloadFile(resourcePath, outputFilePath);
				displaySuccessMessage();
			} else {
				displayErrorMessage("Output file not specified.");			
			}
		} catch (WorkspaceResourceNotFoundException rnf){
			displayErrorMessage("Workspace file not found:  "+resourcePath);			
		} catch (IOException ioe){
			displayErrorMessage("IO Error:  "+ioe.getLocalizedMessage());
		}
	}
	
	private static void doRepositoryUpdateContentCommand(Session session, String resourcePath, String inputFilePath,String comment) {	
		try{
			ResourceCheckinSpecification ciSpec = new ResourceCheckinSpecification( comment);
			
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().updateFileContents(resourcePath, ciSpec ,inputFilePath);				
			displaySuccessMessage();
		} catch (ResourceNotFoundException rnf){
			displayErrorMessage("Resource Not Found:  "+resourcePath);			
		} catch (RepositoryResourceVersioningException rve){
			displayErrorMessage(rve.getLocalizedMessage());			
		} catch (RepositoryResourceCheckoutException rco){
			displayErrorMessage(rco.getLocalizedMessage());			
		} catch (IOException ioe){
			displayErrorMessage(ioe.getLocalizedMessage());
		}
	}
	
	private static void doRepositoryUpdateVersionedContentCommand(Session session, String resourcePath, String inputFilePath, boolean enableVersioningForNewFiles, String versionType, String customVersion, String comment) {	
		try{
			VersionType vt = VersionType.MINOR;
			if(versionType.equalsIgnoreCase("MAJOR")) vt = VersionType.MAJOR;
			if(versionType.equalsIgnoreCase("CUSTOM")) vt = VersionType.CUSTOM;
			ResourceCheckinSpecification ciSpec = new ResourceCheckinSpecification( comment, vt, customVersion);

			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().updateFileContents(resourcePath, ciSpec, inputFilePath);				
			displaySuccessMessage();
		} catch (ResourceNotFoundException rnf){
			displayErrorMessage("Resource Not Found:  "+resourcePath);			
		} catch (RepositoryResourceVersioningException rve){
			displayErrorMessage(rve.getLocalizedMessage());			
		} catch (RepositoryResourceCheckoutException rco){
			displayErrorMessage(rco.getLocalizedMessage());			
		} catch (IOException ioe){
			displayErrorMessage(ioe.getLocalizedMessage());
		}
	}

	private static void doGetFileToWorkspaceCommand(Session session, String path, String versionLabel) {	
		try{
			logger.debug("Get repository file to workspace: "+path+" version?:"+versionLabel);
			if(versionLabel == null || versionLabel.length()<1){
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getCurrentFileVersionToWorkspace(path);
			} else {
				((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getFileVersionToWorkspace(path, versionLabel);
			}
			displaySuccessMessage();
		} catch (IOException ioe){
			displayErrorMessage("I/O error during operation:  "+ioe.getLocalizedMessage());			
		} catch (IllegalPathException e) {
			displayErrorMessage("Illegal path given:  "+e.getLocalizedMessage());			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		}
	}
	
	private static void doCreateRepositoryFolderCommand(Session session, String containerPath, boolean createParentContainers) {	
		try{
			logger.debug("About to attempt creation of a container: "+containerPath);
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().createFolder(containerPath,createParentContainers);
			displaySuccessMessage();
		} catch (ResourceExistsException ree){
			displayErrorMessage("Resource Already Exists:  "+containerPath);			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Parent resource not found:  "+e.getLocalizedMessage());			
		}
	}

	private static void doCreateWorkspaceFolderCommand(Session session, String containerPath, boolean createParentContainers) {	
		try{
			logger.debug("About to attempt creation of a container: "+containerPath);
			((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().createFolder(containerPath,createParentContainers);
			displaySuccessMessage();
		} catch (ResourceExistsException ree){
			displayErrorMessage("Resource Already Exists:  "+containerPath);			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Parent resource not found:  "+e.getLocalizedMessage());			
		}
	}

	private static void doWorkspaceCreateFileCommand(Session session, String path, String localFilePath, boolean createParentContainers) {	
	try{
		((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().createFile(path, localFilePath,createParentContainers);
		displaySuccessMessage();
	} catch (WorkspaceResourceExistsException ree){
		displayErrorMessage("Resource Already Exists:  "+path);			
	} catch (WorkspaceResourceNotFoundException e) {
		displayErrorMessage("Parent resource not found:  "+e.getLocalizedMessage());			
	} catch (FileNotFoundException e) {
		displayErrorMessage("Specified inputfile not found:  "+localFilePath);	
	} catch (IOException e) {
		displayErrorMessage("IO error: "+e.getLocalizedMessage());
	}
}

	private static void doRepositoryCreateFileCommand(Session session, String path, String sourceFilePath, String comment, boolean createParentContainers) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().createFile(path,  sourceFilePath,comment,createParentContainers);
			displaySuccessMessage();
		} catch (ResourceExistsException ree){
			displayErrorMessage("Resource Already Exists:  "+path);			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Parent resource not found:  "+e.getLocalizedMessage());			
		} catch (FileNotFoundException e) {
			displayErrorMessage("Specified inputfile not found:  "+sourceFilePath);	
		} catch (IOException e) {
			displayErrorMessage("IO error: "+e.getLocalizedMessage());
		}
	}

	private static void doCreateVersionedFileCommand(Session session, String targetUri, String sourceFilePath, String startingVersion, String comment, boolean createParentContainers) {	
		try{
			
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().createVersionedFile(targetUri, sourceFilePath, comment, startingVersion,createParentContainers);
			displaySuccessMessage();
		} catch (ResourceExistsException ree){
			displayErrorMessage("Resource Already Exists:  "+targetUri);			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Parent resource not found:  "+e.getLocalizedMessage());			
		} catch (FileNotFoundException e) {
			displayErrorMessage("Specified inputfile not found:  "+sourceFilePath);	
		} catch (IOException e) {
			displayErrorMessage("IO error: "+e.getLocalizedMessage());
		}
	}

	private static void doRepositoryDeleteCommand(Session session, String resourcePath) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().delete(resourcePath);
			displaySuccessMessage();
		} catch (ResourceDeleteException rde){
			displayErrorMessage("Delete exception:  "+rde.getLocalizedMessage());			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		}
	}

	private static void doWorkspaceDeleteCommand(Session session, String resourcePath) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().delete(resourcePath);
			displaySuccessMessage();
		} catch (ResourceDeleteException rde){
			displayErrorMessage("Delete exception:  "+rde.getLocalizedMessage());			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		}
	}

	private static void doCheckOutToWorkspaceCommand(Session session,  String resourcePath) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().checkoutFileToWorkspace( resourcePath);
			displaySuccessMessage();
		} catch (RepositoryResourceCheckoutException rde){
			displayErrorMessage("Checkout exception:  "+rde.getLocalizedMessage());			
		} catch (RepositoryResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		}
	}

	private static void doCheckinFromWorkspaceCommand(Session session, String resourcePath, boolean enableVersioningForNewFiles, String versionType, String customVersion, boolean checkoutInPlace, String comment) {	
		try{
			VersionType vt = VersionType.MINOR;
			if(versionType.equalsIgnoreCase("MAJOR")) vt = VersionType.MAJOR;
			if(versionType.equalsIgnoreCase("CUSTOM")) vt = VersionType.CUSTOM;
			
			ResourceCheckinSpecification ciSpec = new ResourceCheckinSpecification(comment, vt, customVersion);
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().checkinFileFromWorkspace( resourcePath,ciSpec);
			displaySuccessMessage();
		} catch (RepositoryResourceCheckinException rde){
			displayErrorMessage(rde.getLocalizedMessage());			
		} catch (RepositoryResourceNotFoundException e) {
			displayErrorMessage(e.getLocalizedMessage());		
		}
	}
	
	private static void doUndoCheckOutCommand(Session session, String resourcePath) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().undoCheckout( resourcePath);
			displaySuccessMessage();
		} catch (RepositoryResourceUndoCheckoutException rde){
			displayErrorMessage("UndoCheckout exception:  "+rde.getLocalizedMessage());			
		}
	}

	private static void doDisableVersioningCommand(Session session, String resourcePath, String comment) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().disableVersioningOnFile(resourcePath,comment);
			displaySuccessMessage();
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		} catch (RepositoryResourceVersioningException e) {
			displayErrorMessage("Resource versioning exception:  "+e.getLocalizedMessage());			
		}
	}

	private static void doEnableVersioningCommand(Session session, String resourcePath, String comment, String startingVersion) {	
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().enableVersioningOnFile(resourcePath,comment,startingVersion );
			displaySuccessMessage();
		} catch (RepositoryResourceVersioningException e) {
			displayErrorMessage("Resource versioning exception:  "+e.getLocalizedMessage());			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		}
	}
	
	private static void doSaveAndCheckinVersionedContentCommand(Session session, String sddPath, String localFilePath, boolean enableVersioningForNewFiles, String versionType, String customVersion, String comment){
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().updateContentInWorkspaceAndCheckin( sddPath, localFilePath, new ResourceCheckinSpecification( comment,ResourceCheckinSpecification.toVersionType(versionType), customVersion));
			displaySuccessMessage();
		} catch (RepositoryResourceVersioningException e) {
			displayErrorMessage("Resource versioning exception:  "+e.getLocalizedMessage());			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		} catch (RepositoryResourceCheckoutException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (RepositoryResourceCheckinException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (IOException e) {
			displayErrorMessage(e.getLocalizedMessage());
		}	}
	
	private static void doWorkspaceUploadFileCommand(Session session, String sddPath, String localFilePath){
		try{
			((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().updateFileContents( sddPath, localFilePath);
			displaySuccessMessage();
		} catch (WorkspaceResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		} catch (IOException e) {
			displayErrorMessage(e.getLocalizedMessage());
		}		
	}
	
	private static void doSaveAndCheckinContentCommand(Session session, String sddPath, String localFilePath, String comment){
		try{
			ResourceCheckinSpecification ciSpec = new ResourceCheckinSpecification(comment);
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().updateContentInWorkspaceAndCheckin( sddPath, localFilePath,ciSpec);
			displaySuccessMessage();
		} catch (RepositoryResourceVersioningException e) {
			displayErrorMessage("Resource versioning exception:  "+e.getLocalizedMessage());			
		} catch (ResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		} catch (RepositoryResourceCheckoutException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (RepositoryResourceCheckinException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (IOException e) {
			displayErrorMessage(e.getLocalizedMessage());
		}	
	}

	private static void doCheckoutAndGetContentCommand(Session session, String path, String localFilePath){
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().checkoutFileToWorkspaceAndDownload( path, localFilePath);
			displaySuccessMessage();
		} catch (RepositoryResourceNotFoundException e) {
			displayErrorMessage("Repository esource not found:  "+e.getLocalizedMessage());			
		} catch (WorkspaceResourceNotFoundException e) {
			displayErrorMessage("Workspace resource not found:  "+e.getLocalizedMessage());			
		} catch (RepositoryResourceCheckoutException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (IOException e) {
			displayErrorMessage(e.getLocalizedMessage());
		}	
	}
	
	
	private static void doGetSpecifiedVersionContentCommand(Session session, String sddPath, String versionLabel, String localFilePath){
		try{
			((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().downloadFile(sddPath, versionLabel, localFilePath);
			displaySuccessMessage();
		} catch (RepositoryResourceNotFoundException e) {
			displayErrorMessage("Resource not found:  "+e.getLocalizedMessage());			
		} catch (IOException e) {
			displayErrorMessage("I/O error during processing of this command: "+e.getLocalizedMessage());
		}	
	}

	
	
	/* You could do this command in a bunch of ways with the raw API.  In this implementation: 
	 * 
	 *        1) showChildren determines if we are listing this file or just its children.  
	 *        2) Verbose decides how many properties/attributes to show for each resource being listed.
	 */
    private static void doRepositoryListCommand(Session session, String listPath, boolean verbose, boolean showChildren){
        try {                       
            RepositoryResource selection = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getResource(listPath);

            if(!showChildren){
                //Let's print some characteristics of the specified resource
                System.out.println("path: "+selection.getPath());
                if(selection instanceof RepositoryFile){
                    RepositoryFile file = (RepositoryFile) selection;
                    System.out.println("   is versioned: "+file.isVersioned());
                    System.out.println("   versionLabel: "+file.getVersionLabel());
                    System.out.println("   is checkedOut: "+file.isCheckedOut());
                }
                if(verbose){ //let's show more properties AND all attributes on this then.
                    //System.out.println("   type: "+selection.getType());
                //  System.out.println("   subtype: "+selection.getSubtype());
                    if(selection instanceof RepositoryFile){
                        RepositoryFile file = (RepositoryFile) selection;
                        System.out.println("   size: "+file.getSize());
                    }

                    List<Attribute> parentAttrs = selection.getAttributes();
                    for(Attribute attr:parentAttrs){
                        System.out.println("       Attribute "+attr.getName()+ " has value: "+attr.getValue());
                    }
                }
            }

            if(showChildren){
                List<RepositoryResource> kids = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().getChildren(listPath);

                System.out.println(""+kids.size()+" children of "+listPath+" found.");
                for(RepositoryResource child:kids){
                    
                    System.out.println("path: "+child.getPath());
                    if(selection instanceof RepositoryFile){
                        RepositoryFile file = (RepositoryFile) selection;
                        System.out.println("   is versioned: "+file.isVersioned());
                        System.out.println("   versionLabel: "+file.getVersionLabel());
                        System.out.println("   is checkedOut: "+file.isCheckedOut());
                    }
                    if(verbose){
                        //System.out.println("   type: "+child.getType());
                    //  System.out.println("   subtype: "+child.getSubtype());
                        if(selection instanceof RepositoryFile){
                            RepositoryFile file = (RepositoryFile) selection;
                            System.out.println("   size: "+file.getSize());
                        }                       
                        List<Attribute> childAttrs = child.getAttributes();
                        for(Attribute attr:childAttrs){
                            System.out.println("      Attribute "+attr.getName()+" has value: "+attr.getValue());
                        }
                    }
                    System.out.println("    ");
                }
            }
            displaySuccessMessage();
        } catch (ResourceNotFoundException e) {
            displayErrorMessage(e.getLocalizedMessage());
        }       
    }
    

    private static void doWorkspaceListCommand(Session session, String listPath, boolean verbose, boolean showChildren){
        try {                       
            WorkspaceResource selection = ((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().getResource(listPath);
            

            if(!showChildren){
                //Let's print some characteristics of the specified resource
                System.out.println("path: "+selection.getPath());

                if(verbose){ //let's show more properties AND all attributes on this then.
                    if(selection instanceof WorkspaceFile){
                        WorkspaceFile file = (WorkspaceFile) selection;
                        System.out.println("type: file");
                        System.out.println("size: "+file.getSize());
                    } else {
                        System.out.println("type: folder");
                    }
                }
            }

            if(showChildren){
                List<WorkspaceResource> kids = ((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().getChildren(listPath);

                System.out.println(""+kids.size()+" children of "+listPath+" found.");
                for(WorkspaceResource child:kids){
                    
                    System.out.println("path: "+child.getPath());

                    if(verbose){
                        if(child instanceof WorkspaceFile){
                            WorkspaceFile file = (WorkspaceFile) child;
                            System.out.println("   type: file");
                            System.out.println("   size: "+file.getSize());
                        } else {
                            System.out.println("   type: folder");
                        }                       
                    System.out.println("    ");
                    }
                }
            }           
            displaySuccessMessage();
        } catch (WorkspaceResourceNotFoundException e) {
            displayErrorMessage(e.getLocalizedMessage());
        }       
    }
	
	private static void doRepositoryCopyCommand(Session session, String sourcePath, String targetPath){
		try {
			/*RepositoryResource result =*/ ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().copy(sourcePath, targetPath);
			displaySuccessMessage();
		} catch (RepositoryResourceNotFoundException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (RepositoryResourceExistsException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (RepositoryResourceCopyException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (Throwable e) {
			displayErrorMessage("Unexpected error: "+e.getLocalizedMessage());
		}			
}
	private static void doWorkspaceCopyCommand(Session session, String sourcePath, String targetPath){
		try {
			/*WorkspaceResource result =*/ ((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().copy(sourcePath, targetPath);
			displaySuccessMessage();
		} catch (WorkspaceResourceNotFoundException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (WorkspaceResourceCopyException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (WorkspaceResourceExistsException e) {
			displayErrorMessage(e.getLocalizedMessage());
		} catch (Throwable e) {
			displayErrorMessage("Unexpected error: "+e.getLocalizedMessage());
		}			
}

	private static void doProjectExistsCommand(Session session, String userid, String projectName, String orgName){
		try{
			//When the API allows multiple orgs, we would instead ask the org service for the org with name "orgName"
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);

			boolean exists = ((DrugServiceManager) session.getServiceManager()).getProjectService().projectExists(org, projectName);
			System.out.println("Project "+projectName+" exists = "+exists);
			displaySuccessMessage();
		} catch (Throwable e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doProjectGetByNameCommand(Session session, String userid, String projectName, String orgName){
		try{
			//When the API allows multiple orgs, we would instead ask the org service for the org with name "orgName"
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);

			Project proj =  ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org, projectName);
			displayProject(proj);
			displaySuccessMessage();
		} catch (Throwable e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doProjectDeleteCommand(Session session, String userid, String projectName, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);
			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org, projectName);
			((DrugServiceManager) session.getServiceManager()).getProjectService().deleteProject(proj);			
			displaySuccessMessage();
		} catch (ProjectNotFoundException e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		} catch (OrganizationNotFoundException e) {
			displayErrorMessage("Organization not found: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doProjectCreateCommand(Session session, String userid, String projectName, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);

			//Note that you can not set the lead for the project during creation.  If the lead is to be anybody other than
			//the creator, you must first add that ID to the membership of the project before you can identify them as the lead
			//for the project.  
			
			Project proj =	((DrugServiceManager) session.getServiceManager()).getProjectService().createProject(org, projectName,null);
			
			displayProject(proj);
			displaySuccessMessage();
		} catch (OrganizationNotFoundException e) {
			displayErrorMessage("Specified organization not found: "+e.getLocalizedMessage());
		} catch (ProjectExistsException e) {
			displayErrorMessage("Project already exists: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doProjectGetByIDCommand(Session session, String userid, String projectID){
		try{

			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByIdentifier(projectID);
			displayProject(proj);
			displaySuccessMessage();
		} catch (Throwable e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		}	
	}
	
	
	private static void displayProject(Project proj){
		System.out.println("Name: "+proj.getName());
		System.out.println("Lead: "+proj.getLead());
		System.out.println("ID: "+proj.getIdentifier());
		System.out.println("State: "+proj.getState());
		System.out.println("StateComment: "+proj.getStateComment());
		System.out.println("Context: "+proj.getContext().getName());
		System.out.println("Path: "+proj.getPath());
		System.out.println("ObjectType: "+proj.getObjectType());
		
		List<Attribute> childAttrs = proj.getAttributes(); 
		for(Attribute attr:childAttrs){
			System.out.println("      Attribute "+attr.getName()+": "+attr.getValue());
		}
	}
	
	private static void doDisplayProjectsByOrg(Session session, String userid, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);

			List <Project> projs = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectsByOrganization(org);
			System.out.println("There are "+projs.size()+" projects in organization "+orgName);
			for(Project p:projs){
				System.out.println("\n\n");
				displayProject(p);
			}
			displaySuccessMessage();
		} catch (OrganizationNotFoundException e) {
			displayErrorMessage("Organization not found: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doAnalysisExistsCommand(Session session, String userid, String analysisName, String projectName, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);

			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org, projectName);
			boolean exists = ((DrugServiceManager) session.getServiceManager()).getAnalysisService().analysisExists(proj, analysisName);
			System.out.println("Analysis "+analysisName+" exists = "+exists);
			displaySuccessMessage();
		} catch (Throwable e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doAnalysisGetByNameCommand(Session session, String userid, String analysisName, String projectName, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);
			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org, projectName);
			Analysis analysis = ((DrugServiceManager) session.getServiceManager()).getAnalysisService().getAnalysisByName(proj, analysisName);
			displayAnalysis(analysis);
			displaySuccessMessage();
		} catch (Throwable e) {
			displayErrorMessage("Analysis not found: "+e.getLocalizedMessage());
		}	
	}
	

	
	private static void doAnalysisDeleteCommand(Session session, String userid, String analysisName,  String projectName, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);
			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org, projectName);
			Analysis analysis = ((DrugServiceManager) session.getServiceManager()).getAnalysisService().getAnalysisByName(proj, analysisName);
			((DrugServiceManager) session.getServiceManager()).getAnalysisService().deleteAnalysis(analysis);				
			displaySuccessMessage();
		} catch (ProjectNotFoundException e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		} catch (AnalysisNotFoundException e) {
			displayErrorMessage("Analysis not found: "+e.getLocalizedMessage());
		} catch (OrganizationNotFoundException e) {
			displayErrorMessage("Organization not found: "+e.getLocalizedMessage());
		}	
	}

	private static void doAnalysisCreateCommand(Session session, String userid, String analysisName, String projectName, String stateString, String orgName, String comment){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);
			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org, projectName);

			//Note that you can not set the lead for the project during creation.  If the lead is to be anybody other than
			//the creator, you must first add that ID to the membership of the project before you can identify them as the lead
			//for the project.  
	 
			Analysis analysis =	((DrugServiceManager) session.getServiceManager()).getAnalysisService().createAnalysis(proj, analysisName,null);
			
			displayAnalysis(analysis);
			displaySuccessMessage();
		} catch (OrganizationNotFoundException e) {
			displayErrorMessage("Specified organization not found: "+e.getLocalizedMessage());
		} catch (AnalysisExistsException e) {
			displayErrorMessage("Analysis already exists: "+e.getLocalizedMessage());
		} catch (ProjectNotFoundException e) {
			displayErrorMessage("Specified project not found: "+e.getLocalizedMessage());
		}	
	}
	
	private static void doAnalysisGetByIDCommand(Session session, String userid, String analysisID){
		try{
			Analysis analysis  = ((DrugServiceManager) session.getServiceManager()).getAnalysisService().getAnalysisByIdentifier(analysisID);
			displayAnalysis(analysis);
			displaySuccessMessage();
		} catch (Throwable e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());
		}	
	}
	
	
	private static void displayAnalysis(Analysis analysis){
		System.out.println("Name: "+analysis.getName());
		System.out.println("Lead: "+analysis.getLead());
		System.out.println("ID: "+analysis.getIdentifier());
		System.out.println("State: "+analysis.getState());
		System.out.println("StateComment: "+analysis.getStateComment());
		System.out.println("Context: "+analysis.getContext().getName());
		System.out.println("Path: "+analysis.getPath());
		System.out.println("ObjectType: "+analysis.getObjectType());
		
		List<Attribute> childAttrs = analysis.getAttributes(); 
		for(Attribute attr:childAttrs){
			System.out.println("      Attribute "+attr.getName()+": "+attr.getValue());
		}
	}
	
	private static void doDisplayAnalysesByProject(Session session, String userid, String projectName, String orgName){
		try{
			Organization org =  ((DrugServiceManager) session.getServiceManager()).getOrganizationService().getOrganizationByName(orgName);

			Project proj = ((DrugServiceManager) session.getServiceManager()).getProjectService().getProjectByName(org,projectName);
			List <Analysis> pds = ((DrugServiceManager) session.getServiceManager()).getAnalysisService().getAnalysesByProject(proj);
			System.out.println("There are "+pds.size()+" analyses in project "+projectName);
			for(Analysis a:pds){
                System.out.println("\n\n");
                 displayAnalysis(a);
			}
			displaySuccessMessage();
		} catch (OrganizationNotFoundException e) {
			displayErrorMessage("Organization not found: "+e.getLocalizedMessage());
		} catch (ProjectNotFoundException e) {
			displayErrorMessage("Project not found: "+e.getLocalizedMessage());		}	
	}
	
		
	private static void doUserCreateCommand(Session session, String userId, String displayName,String email, String password){
		try {
			((DrugServiceManager) session.getServiceManager()).getUserService().createUser(userId, displayName, email, password);
			displaySuccessMessage();
		} catch (UserExistsException e) {
			displayErrorMessage("User already exists: "+e.getLocalizedMessage());			
		} catch (UserPasswordException e) {
			displayErrorMessage("Password is invalid: "+e.getLocalizedMessage());			
		}
}
	private static void doUserExistsCommand(Session session, String userId){
		boolean answer = ((DrugServiceManager) session.getServiceManager()).getUserService().userExists(userId);
		System.out.println("User exists = "+answer);
}
	private static void doUserDeleteCommand(Session session, String userId){
		try {
			((DrugServiceManager) session.getServiceManager()).getUserService().deleteUser(userId);
			displaySuccessMessage();
		} catch (UserNotFoundException e) {
			displayErrorMessage("User not found: "+e.getLocalizedMessage());
		} catch (UserDeleteException e) {
			displayErrorMessage("Deletion error: "+e.getLocalizedMessage());
		}
}
	private static void doUserGetCommand(Session session, String userId){
		User user;
		try {
			user = ((DrugServiceManager) session.getServiceManager()).getUserService().getUserByUserId(userId);
			displayUser(user);
			displaySuccessMessage();
		} catch (UserNotFoundException e) {
			displayErrorMessage("User not found: "+e.getLocalizedMessage());
		}
}
	private static void doUserResetPasswordCommand(Session session, String userId,String newPassword){
		try {
			((DrugServiceManager) session.getServiceManager()).getUserService().resetPassword(userId, newPassword);
			displaySuccessMessage();
		} catch (UserNotFoundException e) {
			displayErrorMessage("User not found: "+e.getLocalizedMessage());
		} catch (UserPasswordException e) {
			displayErrorMessage("Password is invalid: "+e.getLocalizedMessage());			
		}
}
	
private static void displayUser(User user){
	System.out.println("User name "+user.getDisplayName());
	System.out.println("Userid "+user.getUserId());
	System.out.println("Email "+user.getEmail());
	System.out.println("Account info "+user.getAccountInfo());
}
	
	
	
	
	
	private static void showUsage(){
		System.out.println("Usage: SASDrugDevCommand [-s <server URL>] [-u <username>] [-p <password>] <command> [<command inputs>]");
		System.out.println("where commands are:\n");
		System.out.println("  -help     ");
		System.out.println("     Shows this list of commands in the sample program.");
		System.out.println("  -test     ");
		System.out.println("     Tests if you can talk to the server-side of the SDD java API.");
		System.out.println("                                                         ");
		System.out.println("  -repository_list <repositoryPath> [-verbose] [-showChildren]");
		System.out.println("     Displays information about resources and resource attributes stored in the SDD repository.");
		System.out.println("  -workspace_list <workspacePath> [-verbose] [-showChildren]");
		System.out.println("     Displays information about folders and files stored in your SDD workspace.");
		System.out.println("                                                         ");	
		System.out.println("  -repository_resource_exists <repositoryPath> ");
		System.out.println("     Tells if the specified resource exists in the SDD repository.");
		System.out.println("  -workspace_resource_exists <workspacePath> ");
		System.out.println("     Tells if the specified resource exists in your SDD workspace.");
		System.out.println("                                                         ");	
		System.out.println("  -create_repository_folder <repositoryPath>");
		System.out.println("     Create the specified folder in the SDD repository.");
		System.out.println("  -create_repository_file <repositoryPath> <local file path> [-comment \"Comment about this change.\"] ");
		System.out.println("     Create the specified unversioned file in the SDD repository.");
		System.out.println("  -create_versioned_file <repositoryPath> <local file path> [-startingVersion <initial version number>] [-createParentContainers true] [-comment \"Comment about this file.\"]");
		System.out.println("     Create the specified versioned file in SDD. If no starting version is specified, the default version will be 1.0.");
		System.out.println("  -create_workspace_folder <workspacePath>");
		System.out.println("     Create the specified folder in the SDD workspace.");
		System.out.println("  -create_workspace_file <workspacePath> <local file path>  [-createParentContainers true(default)|false]");
		System.out.println("     Create the specified file in your SDD workspace.");
		System.out.println("                                                         ");	
		System.out.println("  -delete_from_repository <repositoryPath>");
		System.out.println("     Delete the specified file or container from the SDD repository.");
		System.out.println("  -delete_from_workspace <workspacePath>");
		System.out.println("     Delete the specified file or folder from your SDD workspace.");
		System.out.println("                                                         ");
		System.out.println("  -enable_versioning <repositoryPath> [-startingVersion <initial version number>]  [-comment \"Comment about this change.\"] ");
		System.out.println("     Enable versioning for the specified resource in SDD. If no starting version is specified, the default version will be 1.0.");
		System.out.println("  -disable_versioning <repositoryPath>  [-comment \"Comment about this change.\"] ");
		System.out.println("     Disable versioning for the specified resource in SDD.");
		System.out.println("                                                         ");
		System.out.println("  -get_versions <repositoryPath>  ");
		System.out.println("     Returns a list of all stored versions of the specified resource in the SDD repository.");
		System.out.println("                                                         ");
		System.out.println("  -download_repository_file <repositoryPath> <local file path>  ");
		System.out.println("     Write the (latest) content of the specified resource to the local file named.");
		System.out.println("  -download_workspace_file <workspacePath> <local file path>  ");
		System.out.println("     Write the content of the specified file to the local file named.");
		System.out.println("  -download_file_version <repositoryPath> <local file path> [-versionLabel <version number>] ");
		System.out.println("     Write the content of the specified version of the repository file to the local file. If no version is specified, it will get the latest version.");
		System.out.println("                                                         ");			
		System.out.println("  -get_repository_file_to_workspace <repositoryPath> [ -versionLabel <versionLabel>] ");
		System.out.println("     Get the specified repository file to the workspace.");
		System.out.println("                                                         ");			
		System.out.println("  -update_repository_file <repositoryPath> <local file path>  ");
		System.out.println("     Replace the content of the specified unversioned repository resource with the contents of the local file named.");
		System.out.println("  -update_versioned_file <repositoryPath> <local file path> [-versionType <MAJOR|MINOR|CUSTOM> [-customVersion <version number>]]  [-comment \"Comment about this change.\"]");
		System.out.println("     Create a new version of the specified repository resource with the contents of the local file.");
		System.out.println("  -update_workspace_file <workspacePath> <local file path>");
		System.out.println("     Save the content of the local file into the workspace file specified by path.");
		System.out.println("                                                         ");
		System.out.println("  -checkout_file_to_workspace <repositoryPath>");
		System.out.println("     Check out the specified repository file to the workspace.");
		System.out.println("  -checkin_file_from_workspace <repositoryPath> [-versionType <MAJOR|MINOR|CUSTOM> [-customVersion <version number>]]  [-comment \"Comment about this change.\"]");
		System.out.println("     Check in the specified repository file from the workspace.");
		System.out.println("  -undo_checkout <repositoryPath>                       ");
		System.out.println("     Undo the checkout of a repository file.   ");
		System.out.println("                                                         ");
		System.out.println("  -save_content_and_checkin_file <repositoryPath> <local file path> ");
		System.out.println("     Save the content of the local file into the workspace file specified by path and then check it into the repository. It is assumed the repository file is checked out.");
		System.out.println("  -save_content_and_checkin_versioned_file <repositoryPath> <local file path> -versionType <MAJOR|MINOR|CUSTOM> [-customVersion <version number>] [-comment \"Comment...\"]");
		System.out.println("     Save the content of the local file into the workspace file specified by path, and then check this versioned file into the repository. It is assumed the repository file is checked out. If no version is specified, it will check in a minor version.");
		System.out.println("  -checkout_file_and_get_content <repositoryPath> <local file path> ");
		System.out.println("     Check out the specified repository file into the workspace, and then copy that content to the local file specified.");
		System.out.println("                                                         ");	
		System.out.println("  -project_exists projname -org orgName                                      ");	
		System.out.println("      Tells if the specified project exists (under given organization)                   ");	
		System.out.println("  -project_get_by_id projectID                                                 ");	
		System.out.println("     Display project information for the given project ID.                            ");	
		System.out.println("  -project_get_by_name projectName -org orgName                              ");	
		System.out.println("     Display project information for project with the given name and parent organization.  ");	
		System.out.println("  -projects_get_by_org orgName                                               ");	
		System.out.println("     Display all projects associated with a given organization.                  ");	
		System.out.println("  -project_delete projectName -org orgName                                                      ");	
		System.out.println("     Delete the specified project.                                                    ");	
		System.out.println("  -project_create ProjectName -org parentOrg                                  ");	
		System.out.println("     Creates project with the given name.                                                 ");	
		System.out.println("                                                         ");	
		System.out.println("                                                         ");	
  		System.out.println("  -analysis_exists analysisName -project projname -org orgName                                      ");	
		System.out.println("      Tells if the specified analysis exists in specified project");
		System.out.println("  -analysis_get_by_id analysisID                                                 ");	
		System.out.println("     Display analysis information for the given analysis ID.                            ");	
		System.out.println("  -analysis_get_by_name analysisName  -project projname -org orgName                              ");	
		System.out.println("     Display analysis information for the given analysis name (and parent organization).  ");	
		System.out.println("  -analyses_get_by_project projectName -org orgName                                               ");	
		System.out.println("     Display all analyses associated with a given project.                  ");	
		System.out.println("  -analysis_delete analysisName  -project projname -org orgName                                ");	
		System.out.println("     Delete the specified analysis.                                                    ");	
		System.out.println("  -analysis_create analysisName  -project projname -org parentOrg       ");	
		System.out.println("     Creates analysis with the given name.                                                  ");	
		System.out.println("                                                         ");	
        System.out.println("  -exportAudit <localFilePath> [-userid <userid>] [-n <object name>] [-loc <location>] [-from <from date>] [-to <to date>] [-type <types>]     "); 
        System.out.println("     Return audit records based on the query parameters provided.  Type can include one type or multiple types (separated by commas).                                                "); 
        System.out.println("                                                         ");    
        System.out.println("  -getSubscriptionEvents <objectType>           "); 
        System.out.println("     Returns subscribable events for the object type specified.                                                  "); 
        System.out.println("  -createSubscriptionForUser <path> -event <eventName> -userid <userid> [-cascade]"); 
        System.out.println("     Returns subscribable events for the object type specified.                                                  "); 
        System.out.println("  -getSubscriptionsByUser <userid> "); 
        System.out.println("     Returns subscriptions for the specified user.                                                  "); 
        System.out.println("  -getSubscriptionsByPathAndEvent <repositoryPath> -event <eventName> "); 
        System.out.println("     Returns subscribers for the specified path and event.                                                  "); 
        System.out.println("                                                         ");    
  		System.out.println("  -user_exists userId       ");	
		System.out.println("     Does specified user exist?");
		System.out.println("  -user_get userId                                        ");	
		System.out.println("     Display user information for the given userid.                            ");	
		System.out.println("  -user_delete userId                             ");	
		System.out.println("     Delete the specified user. ");	
		System.out.println("  -user_create userid -userPassword initialPassword -displayname userDisplayName -email user_email                                          ");	
		System.out.println("     Create a new user with the specified parameters.          ");	
		System.out.println("  -user_reset userid -userPassword newPassword                      ");	
		System.out.println("     Reset the specified user's password.  ");	
  
		System.out.println("                                                                  ");	}

	
	
	//the displayBlahMessage methods just output some sort of message and do not log things.
	private static void displaySuccessMessage(){
		System.out.println("ok");
	}
	private static void displayErrorMessage(String msg){
		System.err.println("Error:  "+msg);		
	}
	private static void displayNoSessionErrorMessage(){
		displayErrorMessage("Session not established.  Cannot execute this command.");		
	}

	//the recordBLAH methods display messages AND log information
	private static void recordError(String msg){
		recordError(msg,null);
	}

	//simple little error reporting method.
	private static void recordError(String msg, Throwable t){
		displayErrorMessage(msg);
		if(t != null){
			logger.error(msg,t);
		} else {
			logger.error(msg);
		}
	}
	

	private static char[] readInput(String prompt) {
		char[] ret = new char[] {'\0'};
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
			System.out.print(prompt);
			ret = (in.readLine()).toCharArray();
		} catch (IOException e) {
			System.err.println("Error reading input.");
		}
		return ret;
	}	
}
